<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://blohm.com
 * @since      1.0.0
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/admin
 * @author     Poul Werner Blohm <werner@blohm.com>
 */
class gls_dk_wc_class_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

    private $base;

    private $plugin_data;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $data, $base, $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->base = $base;
		$this->plugin_data = $data;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name.'-bootstrap-style-admin', plugin_dir_url( __FILE__ ) . 'css/bootstrap.min.css', array(), '4.6.0', 'all' );
		wp_enqueue_style( $this->plugin_name.'-fontawesome-style-admin', plugin_dir_url( __FILE__ ) . 'css/font-awesome.min.css', array(), 'FA_VERSION', 'all' );

		wp_enqueue_style( "farbtastic" );
		wp_enqueue_style( "wp-color-picker" );	 

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in gls_dk_wc_class_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The gls_dk_wc_class_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/gls_dk_wc-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name.'-bootstrap-script-admin', plugin_dir_url( __FILE__ ) . 'js/bootstrap.bundle.min.js', array( 'jquery' ), '4.6.0', false );	
		wp_enqueue_media();
		wp_register_script( $this->plugin_name . '-settings-js', plugin_dir_url( __FILE__ ) . 'js/settings.js', array( 'farbtastic', 'jquery' ), '1.0.0' );
		wp_enqueue_script( $this->plugin_name . '-settings-js' );	

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in gls_dk_wc_class_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The gls_dk_wc_class_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/gls_dk_wc-admin.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'gls_dk_wc', $this->plugin_data );	

	}

}
