<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://blohm.com
 * @since             1.0.0
 * @package           gls_dk_wc_class
 *
 * @wordpress-plugin
 * Plugin Name:       GLS DK WC
 * Plugin URI:        https://blohm.com/gls-plugin/
 * Description:       GLS offers shipping solutions in Denmark and worldwide. By using this plugin you can integrate GLS shipping methods in WooCommerce for GLS in Denmark only.
 * Version:           1.0.0
 * Author:            Poul Werner Blohm
 * Author URI:        https://blohm.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       gls_dk_wc
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'GLSDKWC_XX_S_VERSION', '1.0.0' );
define( 'GLSDKWC_PLUGIN_GLS_NAME', 'gls_dk_wc' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-gls_dk_wc-activator.php
 */
function activate_gls_dk_wc() {
    if ( class_exists( 'woocommerce' ) ) {
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-gls_dk_wc-activator.php';
		gls_dk_wc_class_Activator::activate();
    } else {     
		wp_die(__('Sorry, but this plugin requires the Woocommerce Plugin to be installed and active.',GLSDKWC_PLUGIN_GLS_NAME).', <br><a href="' . admin_url( 'plugins.php' ) . '">&laquo; '.__('Return to Plugins',GLSDKWC_PLUGIN_GLS_NAME).'</a>');	
    }	
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-gls_dk_wc-deactivator.php
 */
function deactivate_gls_dk_wc() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-gls_dk_wc-deactivator.php';
	gls_dk_wc_class_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_gls_dk_wc' );
register_deactivation_hook( __FILE__, 'deactivate_gls_dk_wc' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-gls_dk_wc.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_gls_dk_wc() {

	$plugin = new gls_dk_wc_class();
	$plugin->run();

}
run_gls_dk_wc();
