<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://blohm.com
 * @since      1.0.0
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/includes
 * @author     Poul Werner Blohm <werner@blohm.com>
 */
class gls_dk_wc_class {

	public function register_post_type ( $post_type = '', $plural = '', $single = '', $description = '', $options = array() ) {

		if ( ! $post_type || ! $plural || ! $single ) return;

		$post_type = new gls_dk_wc_class_Post_Type( $this->get_plugin_name(), $post_type, $plural, $single, $description, $options );

		return ;

	}

	public function register_taxonomy ( $taxonomy = '', $plural = '', $single = '', $post_types = array(), $taxonomy_args = array() ) {

		if ( ! $taxonomy || ! $plural || ! $single ) return;

		$taxonomy = new gls_dk_wc_class_Taxonomy( $this->get_plugin_name(), $taxonomy, $plural, $single, $post_types, $taxonomy_args );

		return ;

	}

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      gls_dk_wc_class_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	protected $base = "pwb_gls";
	
    protected $plugin_data;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'GLSDKWC_XX_S_VERSION' ) ) {
			$this->version = GLSDKWC_XX_S_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = GLSDKWC_PLUGIN_GLS_NAME;

		$this->load_dependencies();
		$this->set_locale();
		$this->plugin_data = array (
			'get_stylesheet_directory_uri'   => get_stylesheet_directory_uri() , //alert(gls_wc.get_stylesheet_directory_uri);
			'get_template_directory_uri'   => get_template_directory_uri() , //alert(gls_wc.get_template_directory_uri);
			'get_plugin_dir_url'   => plugin_dir_url( __DIR__ ) , //alert(gls_wc.get_plugin_dir_url);
			'ajaxurl' => admin_url('admin-ajax.php'),//alert(gls_wc.ajaxurl);
			'get_plugin_name'   => $this->get_plugin_name() , //alert(gls_wc.get_plugin_name);
			'get_base'   => $this->get_base() , //alert(gls_wc.get_base);
			'Hello_string' => 'Hello', //alert(gls_wc.Hello_string);
			'get_standard_tab_txt' => 'Standard', //alert(gls_wc.get_standard_tab_txt);
			'get_extra_tab_txt' => 'Extra',//alert(gls_wc.get_extra_tab_txt);
			'get_gls_tab_txt' => 'Gls',//alert(Gls_WC.get_extra_tab_txt);
			'get_gls_tab_shop_time' => 'Business_Hour',//alert(gls_wc.get_extra_tab_txt);
			'get_gls_tab_checkout' => 'gls_checkout',//alert(gls_wc.get_gls_tab_checkout);
			'get_gls_diverse_txt' => 'gls_diverse',//alert(gls_wc.get_gls_diverse_txt);
			'gls_checkout_remove_all' => get_option($this->base.$this->plugin_name.'_gls_checkout_wc_gls_checkout_remove_all'),//alert(gls_wc.gls_checkout_remove_all);
			'gls_checkout_remove_pickup' =>get_option($this->base.$this->get_plugin_name().'_gls_checkout_wc_gls_checkout_remove_pickup'),//alert(gls_wc.gls_checkout_remove_pickup);
			'gls_checkout_remove_business_delevery' =>get_option($this->base.$this->get_plugin_name().'_gls_checkout_wc_gls_checkout_remove_business_delevery'),//alert(gls_wc.gls_checkout_remove_business_delevery);
			'gls_checkout_remove_home_delevery' =>get_option($this->base.$this->get_plugin_name().'_gls_checkout_wc_gls_checkout_remove_home_delevery'),//alert(gls_wc.gls_checkout_remove_home_delevery);
			'gls_remove_pickup_at_firm' =>get_option($this->base.$this->get_plugin_name().'_gls_checkout_wc_gls_checkout_remove_pickup_at_firm'),//alert(gls_wc.gls_remove_pickup_at_firm);
			);
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - gls_dk_wc_class_Loader. Orchestrates the hooks of the plugin.
	 * - gls_dk_wc_class_i18n. Defines internationalization functionality.
	 * - gls_dk_wc_class_Admin. Defines all hooks for the admin area.
	 * - gls_dk_wc_class_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-gls_dk_wc-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-gls_dk_wc-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-gls_dk_wc-admin.php';
		
		require_once(plugin_dir_path( dirname( __FILE__ ) ).'includes/fpdf182/fpdf.php');
		require_once(plugin_dir_path( dirname( __FILE__ ) ).'includes/FPDI/autoload.php');

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */		
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-gls_dk_wc-public.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-gls_dk_wc-settings.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-gls_dk_wc-hooks-admin.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/lib/class-gls_dk_wc-admin-api.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/lib/class-gls_dk_wc-post-type.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/lib/class-gls_dk_wc-taxonomy.php';

		$this->loader = new gls_dk_wc_class_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the gls_dk_wc_class_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new gls_dk_wc_class_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new gls_dk_wc_class_Admin(  $this->get_plugin_data(), $this->get_base(),  $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$plugin_settings = new gls_dk_wc_class_Settings( $this);
		$this->admin = new gls_dk_wc_class_Admin_API($this->get_plugin_name());

		$this->loader->add_action( 'init', $plugin_settings, 'init_settings' , 11 );
		$this->loader->add_action( 'admin_init', $plugin_settings, 'register_settings' );
		$this->loader->add_action( 'admin_menu' , $plugin_settings, 'add_menu_item' );
		$this->loader->add_filter( 'plugin_action_links_' . $this->get_plugin_name() , $plugin_settings, 'add_settings_link' );
		$plugin_action_hooks = new gls_dk_wc_class_hooks_admin( $this );
		$this->loader->add_action( 'wp_ajax_callback_reset_gls_dk_wc', $plugin_action_hooks, 'callback_reset_gls_dk_wc' );
		$this->loader->add_action( 'wp_ajax_nopriv_callback_reset_gls_dk_wc', $plugin_action_hooks, 'callback_reset_gls_dk_wc' );


	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new gls_dk_wc_class_Public( $this );
		
		$this->loader->add_action( 'init', $plugin_public, 'init_settings' , 11 );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    gls_dk_wc_class_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public function get_base() {

		return $this->base;

	}

	public function get_plugin_data() {

		return $this->plugin_data;

	}	
}
