<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://blohm.com
 * @since      1.0.0
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/public
 * @author     Poul Werner Blohm <werner@blohm.com>
 */

use setasign\Fpdi\Fpdi;
use setasign\Fpdi\PdfParser\StreamReader;

class GLSDKWC_PDF_JavaScript extends FPDI {

    var $javascript;
    var $n_js;

    function IncludeJS($script) {
        $this->javascript=$script;
    }

    function _putjavascript() {
        $this->_newobj();
        $this->n_js=$this->n;
        $this->_out('<<');
        $this->_out('/Names [(EmbeddedJS) '.($this->n+1).' 0 R]');
        $this->_out('>>');
        $this->_out('endobj');
        $this->_newobj();
        $this->_out('<<');
        $this->_out('/S /JavaScript');
        $this->_out('/JS '.$this->_textstring($this->javascript));
        $this->_out('>>');
        $this->_out('endobj');
    }

    function _putresources() {
        parent::_putresources();
        if (!empty($this->javascript)) {
            $this->_putjavascript();
        }
    }

    function _putcatalog() {
        parent::_putcatalog();
        if (!empty($this->javascript)) {
            $this->_out('/Names <</JavaScript '.($this->n_js).' 0 R>>');
        }
    }
}

class GLSDKWC_PDF_AutoPrint extends GLSDKWC_PDF_JavaScript
{
    function AutoPrint($dialog=false)
    {
        //Open the print dialog or start printing immediately on the standard printer
        $param=($dialog ? 'true' : 'false');
        $script="print($param);";
        $this->IncludeJS($script);
    }

    function AutoPrintToPrinter($server, $printer, $dialog=false)
    {
        $script = "document.contentWindow.print();";
        $this->IncludeJS($script);
    }
}

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    gls_dk_wc_class
 * @subpackage gls_dk_wc_class/public
 * @author     Poul Werner Blohm <werner@blohm.com>
 */
class gls_dk_wc_class_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

    private $base;

    private $plugin_data;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $parent ) {

		$this->plugin_name = $parent->get_plugin_name();
		$this->version = $parent->get_version();
		$this->base = $parent->get_base();
		$this->plugin_data = $parent->get_plugin_data();
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		
		wp_enqueue_style( $this->plugin_name.'-leaflet', plugin_dir_url( __FILE__ ) . 'css/leaflet.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->plugin_name.'-bootstrap-style-public', plugin_dir_url( __FILE__ ) . 'css/bootstrap.min.css', array(), '4.6.0', 'all' );
		wp_enqueue_style( $this->plugin_name.'-fontawesome-style-public', plugin_dir_url( __FILE__ ) . 'css/font-awesome.min.css', array(), 'FA_VERSION', 'all' );

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in gls_dk_wc_class_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The gls_dk_wc_class_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/gls_dk_wc-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name.'-bootstrap-script-public', plugin_dir_url( __FILE__ ) . 'js/bootstrap.bundle.min.js', array( 'jquery' ), '4.6.0', false );	

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in gls_dk_wc_class_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The gls_dk_wc_class_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name.'-leaflet', plugin_dir_url( __FILE__ ) . 'js/leaflet.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/gls_dk_wc-public.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'gls_dk_wc', $this->plugin_data );	

	}

	function log_me($message) {
		if ( WP_DEBUG === true ) {
			if ( is_array($message) || is_object($message) ) {
				error_log( print_r($message, true) );
			} else {
				error_log( $message );
			}
		}
	}
	
	public function shipping_cost_based_on_api( $rates, $package ){

		$parcelshop_idx = WC()->session->get( '_werner_shop_idx');

		if ( $parcelshop_idx != '3' )
			return $rates;
			
		//if( WC()->session->__isset( 'shipping_cost' ) ) {

			// Loop through the shipping taxes array
			foreach ( $rates as $rate_key => $rate ){
				$has_taxes = false;

				if( 'flat_rate' === $rate->method_id ){
					// Get the initial cost
					$initial_cost = $new_cost = $rates[$rate_key]->cost;

					// Get the new cost
					$new_cost = 0;//WC()->session->get( 'shipping_cost' );

					// Set the new cost
					$rates[$rate_key]->cost = $new_cost;

					// Taxes rate cost (if enabled)
					$taxes = [];
					// Loop through the shipping taxes array (as they can be many)
					foreach ($rates[$rate_key]->taxes as $key => $tax){
						if( $rates[$rate_key]->taxes[$key] > 0 ){
							// Get the initial tax cost
							$initial_tax_cost = $new_tax_cost = $rates[$rate_key]->taxes[$key];
							// Get the tax rate conversion
							$tax_rate    = $initial_tax_cost / $initial_cost;
							// Set the new tax cost
							$taxes[$key] = $new_cost * $tax_rate;
							$has_taxes   = true; // Enabling tax
						}
					}
					if( $has_taxes )
						$rates[$rate_key]->taxes = $taxes;
				}
			}
		//}
		return $rates;
	}

	// Enabling, disabling and refreshing session shipping methods data
	public function refresh_shipping_methods( $post_data ){
		$bool = true;
		//if ( WC()->session->__isset('shipping_cost' ) ) $bool = false;

		// Mandatory to make it work with shipping methods
		foreach ( WC()->cart->get_shipping_packages() as $package_key => $package ){
			WC()->session->set( 'shipping_for_package_' . $package_key, $bool );
		}
		WC()->cart->calculate_shipping();
	}
	
	/**
	 * Initialise settings
	 * @return void
	 */
	public function init_settings () {
		
		$wc_gls_testmode = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_txt'].'_wc_gls_testmode','on');
		//$this->log_me("wc_gls_testmode: $wc_gls_testmode");
		if($wc_gls_testmode != 'on'){	
		
			$gls_username = get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_username');
			if( ! $gls_username || $gls_username == ''){
				return;
			}
			$gls_password= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_password');
			if( ! $gls_password || $gls_password == ''){
				return;
			}
			$gls_Customerid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_Customerid');
			if( ! $gls_Customerid || $gls_Customerid == ''){
				return;
			}
			$gls_contactid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_contactid');
			if( ! $gls_contactid || $gls_contactid == ''){
				return;
			}
		}
		add_action( 'init', array($this,'wc_session_enabler') );
		
		$wc_gls_shipping_setup = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_txt'].'_wc_gls_shipping_setup','on');		
		//$this->log_me("wc_gls_shipping_setup: $wc_gls_shipping_setup");
		if($wc_gls_shipping_setup === 'on'){
			$this->setup_gls_shippings_zones();
		}
		
		add_action( 'wp_ajax_wc_action_callback_gls',array($this,'wc_action_callback_gls') ); //For admin and logged in users
		add_action( 'wp_ajax_nopriv_wc_action_callback_gls', array($this,'wc_action_callback_gls') );//for users
		
		add_action('woocommerce_order_actions', array($this,'custom_wc_order_action'), 10, 1 );
		add_action( 'woocommerce_order_action_custom_action', array($this,'fired_my_custom_action') );
		
		//add_action('woocommerce_admin_field_is_shipping_method_setup', array($this,'is_shipping_method_setup'));
		
		add_filter( 'manage_edit-shop_order_columns', array($this,'sv_wc_cogs_add_order_profit_column_header'), 20 );
		add_action( 'manage_shop_order_posts_custom_column', array($this,'sv_wc_cogs_add_order_profit_column_content') );
		$this->gls_pdf_callback();
		add_action( 'woocommerce_email_customer_details', array($this,'dcwd_add_tracking_info_to_completed_order_email'), 30, 4 );
		add_filter( 'woocommerce_email_attachments', array($this,'webroom_attach_to_wc_emails'), 10, 3);
		add_action( 'woocommerce_checkout_update_order_meta', array($this,'save_custom_field_in_session'));
		add_action( 'woocommerce_checkout_after_customer_details', array($this,'pwb_gls_delivery_options'));	
		add_action( 'woocommerce_after_checkout_validation', array($this,'check_one_err'), 9999, 2);
		add_filter('woocommerce_package_rates', array($this,'shipping_cost_based_on_api'), 12, 2);
		add_action( 'woocommerce_checkout_update_order_review', array($this,'refresh_shipping_methods'), 10, 1 );
		/*
		$wc_gls_add_tracking_info_to_completed_order_email = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_gls_add_tracking_info_to_completed_order_email','on');
		if( $wc_gls_add_tracking_info_to_completed_order_email == 'on' )
			add_action( 'woocommerce_package_rates',array($this,'hide_local_pickup_shipping_methods'), 10, 2 );
		
		$wc_disable_shipping_calc_on_cart= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_diverse_txt'].'_wc_disable_shipping_calc_on_cart');
		if( $wc_disable_shipping_calc_on_cart == 'on' ){
			add_filter( 'woocommerce_cart_ready_to_calc_shipping', array($this,'disable_shipping_calc_on_cart'), 99 );
		}*/
	}
	
	function wc_session_enabler() {
		if ( ! is_admin() && ! WC()->session->has_session() ) {
			WC()->session->set_customer_session_cookie( true );
		}
	}	
	
	function hide_local_pickup_shipping_methods( $rates, $package ) {
		// Loop through shipping methods
		foreach( $rates as $rate_key => $rate ) {
			if( 'local_pickup' === $rate->method_id ) {
				unset($rates[$rate_key]);
			}
		}
		return $rates;
	}

	function disable_shipping_calc_on_cart( $show_shipping ) {
		if( is_cart() ) {
			return false;
		}
		return $show_shipping;
	}		
	
	public function custom_wc_order_action( $actions ) {

		if ( is_array( $actions ) ) {
			$actions['custom_action'] = __( 'Send Email Tracking GLS Number to customer',$this->plugin_name );
		}

		return $actions;
	}
	
	function dcwd_add_tracking_info_to_completed_order_email( $order, $sent_to_admin, $plain_text, $email ) {
		if ( 'customer_completed_order' != $email->id ) {
			return;
		}
		if ( $sent_to_admin ) {
			return;
		}
	

		$gls_value = $order->get_meta( 'gls_order_tracking_number' );

		if ( ! empty($gls_value) ) {

			$wc_gls_add_tracking_info_to_completed_order_email = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_gls_add_tracking_info_to_completed_order_email','on');
			if( $wc_gls_add_tracking_info_to_completed_order_email == 'on' ){			
				if ( $plain_text ) {
					echo '\n**'.__("GLS tracking ID", $this->plugin_name). $gls_value;
				}
				else {
					$gls_url = 'https://gls-group.eu/DK/da/find-pakke';
					$tracking_url = 'https://gls-group.eu/DK/da/find-pakke?match='.$gls_value;
					$message_body      = '<p><strong>'. __("GLS tracking ID", $this->plugin_name).':</strong> ' . $gls_value . '</p>
					<p>'. sprintf( __("They can track their package on %s or directly from %s", $this->plugin_name),'<a href="'.$gls_url.'" target="_blank">'.__("GLS website", $this->plugin_name).'</a>',
					'<a href="'.$tracking_url.'" target="_blank">'.__("GLS number site", $this->plugin_name).$shipping_email.'</a>.</p>');
					echo $message_body;
					//echo '<p><strong>'. __("GLS tracking ID", $this->plugin_name).':</strong> ' . $gls_value . '</p>';
				}
			}
		}
	}
	
	/**
	 * Filter name is woocommerce_order_action_{$action_slug}
	 */
	public function fired_my_custom_action( $order ) {
		
		$gls_value = $order->get_meta( 'gls_order_tracking_number' );

		if ( ! empty($gls_value) ) {
					
			global $woocommerce;
			
			// Create a mailer
			$mailer = $woocommerce->mailer();
			$title        = __("Spor deres ordre","woocommerce",$this->plugin_name);
			$gls_url = 'https://gls-group.eu/DK/da/find-pakke';
			$tracking_url = 'https://gls-group.eu/DK/da/find-pakke?match='.$gls_value;
			$message_body      = '<p><strong>'. __("GLS tracking ID", $this->plugin_name).':</strong> ' . $gls_value . '</p>
			<p>'. sprintf( __("They can track their package on %s or directly from %s", $this->plugin_name),'<a href="'.$gls_url.'" target="_blank">'.__("GLS website", $this->plugin_name).'</a>',
			'<a href="'.$tracking_url.'" target="_blank">'.__("GLS number site", $this->plugin_name).$shipping_email.'</a>.</p>');
			$message = $mailer->wrap_message(sprintf( __( 'Yours Order %s GLS tracking number.',$this->plugin_name ), $order->get_order_number() ), $message_body );
			$ship_email = get_post_meta( $order->get_id(), '_shipping_email', true );
			if( $ship_email != $order->billing_email ) {	
				$mailer->send( $ship_email, sprintf( __( 'Order %s received',$this->plugin_name ), $order->get_order_number() ), $message );
			} else {
				$mailer->send( $order->billing_email, sprintf( __( 'Order %s received',$this->plugin_name ), $order->get_order_number() ), $message );
			}		
		}
	}

	/**
	 * Adds 'Tracking' column header to 'Orders' page immediately after 'order_total' column.
	 *
	 * @param string[] $columns
	 * @return string[] $new_columns
	 */
	public function sv_wc_cogs_add_order_profit_column_header( $columns ) {

		$new_columns = array();

		foreach ( $columns as $column_name => $column_info ) {

			$new_columns[ $column_name ] = $column_info;

			if ( 'order_total' === $column_name ) {
				$new_columns['gls_shipping_information'] = __( 'GLS Shipping Information', $this->plugin_name );
			}
		}

		return $new_columns;
	}

	/**
	 * Adds 'Tracking' column content to 'Orders' page immediately after 'Total' column.
	 *
	 * @param string[] $column name of column being displayed
	 */
	public function sv_wc_cogs_add_order_profit_column_content( $column ) {
		global $post, $pagenow;

		if ( 'gls_shipping_information' === $column ) {

			$order    = wc_get_order( $post->ID );
			$order_id = $order->get_id();
			
			$label           = get_post_meta($order_id, '_gls_label');
			
			if ($label && $label[0]) {
				$tracking_number=$label[0]->Parcels[0]->ParcelNumber;
				$tracking_url = 'https://gls-group.eu/DK/da/find-pakke?match='.$tracking_number;
				 echo __('Track ID', $this->plugin_name) . ": <a href='$tracking_url' target='_blank'>" . $tracking_number. '</a>';
				
					$pdf_link = add_query_arg(array('gls_pdf_action' => 'download', 'post' => $post->ID, '_wpnonce' => wp_create_nonce('download')), admin_url($pagenow));
					echo " | <a href='$pdf_link'>" . __('Download Label',$this->plugin_name) . '</a>';
					
				
					$pdf_link_print = add_query_arg(array('gls_pdf_action' => 'print_label', 'post' => $post->ID, '_wpnonce' => wp_create_nonce('print_label')), admin_url($pagenow));
					echo " | <a href='$pdf_link_print' target='_blank'>" . __('Print Label',$this->plugin_name) . '</a><br />';
			}
		}
	}
	
	public function gls_pdf_callback()
    {
        if (!(isset($_GET['post']) && isset($_GET['gls_pdf_action']))) {
            return;
        }

        // Verify Nonce
        $action = sanitize_key($_GET['gls_pdf_action']);
        $nonce  = sanitize_key($_GET['_wpnonce']);
        if (!wp_verify_nonce($nonce, $action)) {
            wp_die('Invalid request.');
        }

        if(!current_user_can('manage_woocommerce')) {
            wp_die('Access denied');
        }

        $order_id  = intval($_GET['post']);
        //$order_ids = intval($_GET['post_ids']);

        // execute pdf action.
        switch ($action) {
            case 'view':
                $pdf_string = $this->add_pdf_label_to_array(array($order_id));
                $this->view_pdf($pdf_string[0]);
                break;

            case 'download':
                $pdf_string = $this->add_pdf_label_to_array(array($order_id));
                $this->view_pdf($pdf_string[0], 'attachment');
                break;
				
            case 'print_label':
                $this->add_pdf_label_to_array(array($order_id));
                $this->print_pdf($order_id);
                break;

            case 'merge':
                //$this->add_pdf_label_to_array($order_ids);
                //$this->merge_pdf();
                break;
        }
    }
	
	function webroom_attach_to_wc_emails ( $attachments , $email_id, $order ) {
		
		if ( isset(WC()->session) ){
			/*$this->log_me("WC()->session: true");
			if( WC()->session->has_session() ) 
				$this->log_me("WC()->session->has_session: true");
			else
				$this->log_me("WC()->session->has_session: false");*/
		}else{
			//$this->log_me("WC()->session: false");
			return $attachments;
		}
	
		if ( ! is_a( $order, 'WC_Order' ) || ! isset( $email_id ) ) {
			return $attachments;
		}
		$order_id = method_exists( $order, 'get_id' ) ? $order->get_id() : $order->id;
		
		
		$excluded_admin_email_ids = array( 'customer_processing_order' );
		if( in_array( $email_id, $excluded_admin_email_ids ) ) 
			return $attachments;
			
		if( isset( $order ) && is_object( $order ) ){
			$post_id = $order->get_id();
			$this->get_ordre_label_track($post_id);
			$pdf_label_array=array();
			$gls_label = get_post_meta($post_id, $key = '_gls_label');
			if ($gls_label[0]->PDF) {
				$pdf_label_array = base64_decode($gls_label[0]->PDF);
				$pdf = new Fpdi();
				$pdf->setSourceFile(StreamReader::createByString($pdf_label_array));

				$templateId = $pdf->importPage(1);
				$size       = $pdf->getTemplateSize($templateId);

				$pdf->AddPage($size['orientation'], array($size['width'], $size['height']));
				$pdf->useTemplate($templateId);
				$pdf->Output(plugin_dir_path( dirname( __FILE__ ) )."attachments/".$order->get_id().'.pdf', 'F');
				$attachments[] = plugin_dir_path( dirname( __FILE__ ) )."attachments/".$order->get_id().'.pdf';
				}			
		}
		return $attachments;
	}

	function save_custom_field_in_session( $order_id ) {
		$arr=$_REQUEST['properties'];
		$arr=$this->recursive_sanitize_text_field($arr);
	
		$parcelshop_idx=isset( $arr['parcelshop_idx'] )  ? esc_attr( $arr['parcelshop_idx'] )  : '';
		update_post_meta( $order_id, '_parcelshop_idx', $parcelshop_idx );
		
		$parcelshop_number =isset( $arr['parcelshop_number'] ) ? esc_attr( $arr['parcelshop_number'] ) : '';			
		update_post_meta( $order_id, '_parcelshop_number', $parcelshop_number );
		
		$parcelshop_message =isset( $arr['parcelshop_message'] ) ? esc_attr( $arr['parcelshop_message'] ) : '';			
		update_post_meta( $order_id, '_parcelshop_message', $parcelshop_message );
		
		$ship_to_different_address =isset( $arr['ship_to_different_address'] ) ? esc_attr( $arr['ship_to_different_address'] ) : '';			
		update_post_meta( $order_id, '_ship_to_different_address', $ship_to_different_address );
		
		$weight = WC()->cart->get_cart_contents_weight();
		update_post_meta( $order_id, '_cart_weight', $weight );
	}
 
	/**
	 * Recursive sanitation for an array
	 * 
	 * @param $array
	 *
	 * @return mixed
	 */
	function recursive_sanitize_text_field($array) {
		foreach ( $array as $key => &$value ) {
			if ( is_array( $value ) ) {
				$value = recursive_sanitize_text_field($value);
			}
			else {
				$value = sanitize_text_field( $value );
			}
		}

		return $array;
	}		
		
	function check_one_err( $fields, $errors ){
		if ( empty( $errors ) ) {
			return;
		}
		// if any validation errors
		
		$billing_company=$fields['billing_company'];
		$shipping_company=$fields['shipping_company'];
		$billing_email=$fields['billing_email'];
		$billing_phone=$fields['billing_phone'];
		$ship_to_different_address=$fields['ship_to_different_address'];
		
		$val = -1;
		$message= __('Please select shipping method!',$this->plugin_name);
			
		$arr=$_REQUEST['properties'];
		$arr=$this->recursive_sanitize_text_field($arr);
	
		$parcelshop_idx=isset( $arr['parcelshop_idx'] )  ? esc_attr( $arr['parcelshop_idx'] )  : '';
		
		$parcelshop_number =isset( $arr['parcelshop_number'] ) ? esc_attr( $arr['parcelshop_number'] ) : '';
		
		switch( $parcelshop_idx ){
			case "0":
				if( $parcelshop_number != "" )
					$val = 0;
				else
					$message= __('Please select shipping parcel!',$this->plugin_name);
					break;
			case "1":
				if( $ship_to_different_address ){
					if( $shipping_company==""){
						$message= __('Please give Shipping Business name!',$this->plugin_name);
						break;
					}else
					if( $billing_email==""){
						$message= __('Please give Billing email!',$this->plugin_name);
						break;
					}else
					if( $billing_phone==""){
						$message= __('Please give Billing phone number!',$this->plugin_name);
						break;
					}else
						$val = 1;
				}else{
					if( $billing_company==""){
						$message= __('Please give a Billing Business name!',$this->plugin_name);
						break;
					}else
					if( $billing_email==""){
						$message= __('Please give Billing email!',$this->plugin_name);
						break;
					}else
					if( $billing_phone==""){
						$message= __('Please give Billing phone number!',$this->plugin_name);
						break;
					}else
						$val = 1;
				}
					break;
			case "2":
					$val = 2;
					break;
			case "3":
					$val = 3;
					break;
		}
		// add our custom one
		if( $val == -1 )
			$errors->add( 'validation', $message);
	}		
	
    /**
     * @param array $post_ids
     *
     * @return array
     */
    public function add_pdf_label_to_array($post_ids = array())
    {
        foreach ($post_ids as $post_id) {
            if ($post_id) {
                $gls_label = get_post_meta($post_id, $key = '_gls_label');
                if ($gls_label[0]->PDF) {
                    $pdf_label_array[] = base64_decode($gls_label[0]->PDF);
                }
            }
        }

        return $pdf_label_array;
    }

    /**
     * @param        $pdf_string
     * @param string $type
     * @param string $filename
     */
    public function view_pdf($pdf_string, $type = 'inline', $filename = 'gls_label')
    {
        $full_path = $filename . '.pdf';

        header('Content-type: application/pdf');
        header('Content-Disposition: ' . $type . '; filename="' . basename($full_path) . '"');
        header('Content-Transfer-Encoding: binary');
        header('Content-Length: ' . strlen($pdf_string));
        header('Accept-Ranges: bytes');

        echo $pdf_string;
        exit;
    }

    /**
     * @param string $type
     * @throws \setasign\Fpdi\PdfParser\CrossReference\CrossReferenceException
     * @throws \setasign\Fpdi\PdfParser\Filter\FilterException
     * @throws \setasign\Fpdi\PdfParser\PdfParserException
     * @throws \setasign\Fpdi\PdfParser\Type\PdfTypeException
     * @throws \setasign\Fpdi\PdfReader\PdfReaderException
     */
    public function merge_pdf($type = 'inline')
    {
        if (count($this->pdf_label_array) < 1) {
            return;
        }

        $pdf = new Fpdi();

        // iterate through the files
        foreach ($this->pdf_label_array as $pdf_label) {

            $pdf->setSourceFile(StreamReader::createByString($pdf_label));

            $templateId = $pdf->importPage(1);
            $size       = $pdf->getTemplateSize($templateId);

            $pdf->AddPage($size['orientation'], array($size['width'], $size['height']));
            $pdf->useTemplate($templateId);
        }

        $pdf_string = $pdf->Output('S');
        $this->view_pdf($pdf_string, $type);
    }

    public function print_pdf($post_id)
    {
		$gls_label = get_post_meta($post_id, $key = '_gls_label');
		if ($gls_label[0]->PDF) {
			ob_start();
			$pdf_label_string = base64_decode($gls_label[0]->PDF);
		
			$pdf=new GLSDKWC_PDF_AutoPrint();
			$pageCount = $pdf->setSourceFile(StreamReader::createByString($pdf_label_string));

            $templateId = $pdf->importPage(1);
            $size       = $pdf->getTemplateSize($templateId);

            $pdf->AddPage($size['orientation'], array($size['width'], $size['height']));
            $pdf->useTemplate($templateId);
			$wc_gls_auto_print= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_diverse_txt'].'_wc_gls_auto_print');
			if( $wc_gls_auto_print == 'on' )
				$pdf->AutoPrint(true);
			else
				$pdf->AutoPrint();
			ob_end_clean();
			$pdf->Output();
		}
	}
	
	function get_ordre_label_track( $order_id ) {

		if ( ! $order_id )
			return;	
		
		$order = wc_get_order( $order_id );
			
		$parcelshop_idx = get_post_meta( $order_id, '_parcelshop_idx', true );
		//$this->log_me("parcelshop_idx:$parcelshop_idx");
		
		$parcelshop_number = get_post_meta( $order_id, '_parcelshop_number', true );
		//$this->log_me("parcelshop_number:$parcelshop_number");
			
		$parcelshop_message = get_post_meta( $order_id, '_parcelshop_message', true );
		//$this->log_me("parcelshop_message:$parcelshop_message");
			
		$ship_to_different_address = get_post_meta( $order_id, '_ship_to_different_address', true );
		//$this->log_me("ship_to_different_address:$ship_to_different_address");
		
		if( $parcelshop_idx != 3 ){
			$billing_address_1 = "";
			$billing_address_2 = "";
			$billing_city = "";
			$billing_company = "";
			$billing_country = "";
			$billing_email = "";
			$billing_first_name = "";
			$billing_last_name = "";
			$billing_phone = "";
			$billing_postcode = "";

			if ( $order_id > 0 ) {
				if ( $order instanceof WC_Order ) {				
					if( $ship_to_different_address == "shipping_"){
						$billing_address_1 = $order->get_shipping_address_1();
						$billing_address_2 = $order->get_shipping_address_2();
						$billing_city = $order->get_shipping_city();
						$billing_company = $order->get_shipping_company();
						$billing_country = $order->get_shipping_country();
						$billing_first_name = $order->get_shipping_first_name();
						$billing_last_name = $order->get_shipping_last_name();
						$billing_postcode = $order->get_shipping_postcode();
	
						$s_email=$order->get_meta( 'shipping_email' );
						if ( ! empty($s_email) )
							$billing_email = $s_email;  
						else
							$billing_email = $order->get_billing_postcode();
						
						$s_phone=$order->get_meta( 'shipping_phone' );
						if ( ! empty($s_phone) )
							$billing_phone = $s_phone;
						else
							$billing_phone = $order->get_billing_phone();
					}else{
						$billing_address_1 = $order->get_billing_address_1();
						$billing_address_2 = $order->get_billing_address_2();
						$billing_city = $order->get_billing_city();
						$billing_company = $order->get_billing_company();
						$billing_country = $order->get_billing_country();
						$billing_email = $order->get_billing_email();
						$billing_first_name = $order->get_billing_first_name();
						$billing_last_name = $order->get_billing_last_name();
						$billing_phone = $order->get_billing_phone();
						$billing_postcode = $order->get_billing_postcode();
					}	
				}
			}else
			return;
			
			$billing_name = $billing_first_name." ".$billing_last_name;
			$billing__contact = $billing_name." ".$billing_phone;
			$weight = get_post_meta( $order_id, '_cart_weight', true );
			$wc_weight_extra= get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_diverse_txt'].'_wc_weight_extra','0.3');
			$total_weight=$weight+$wc_weight_extra;

			if($order->is_paid())
				$paid = 'yes';
			else
				$paid = 'no';

			// iterating through each order items (getting product ID and the product object) 
			// (work for simple and variable products)
			foreach ( $order->get_items() as $item_id => $item ) {

				if( $item['variation_id'] > 0 ){
					$product_id = $item['variation_id']; // variable product
				} else {
					$product_id = $item['product_id']; // simple product
				}

				// Get the product object
				$product = wc_get_product( $product_id );
			}
			
			$wc_gls_AlternativeShipper= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper','');
			if( $wc_gls_AlternativeShipper == 'on' ){
				$wc_gls_AlternativeShipper_name= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_name');
				$wc_gls_AlternativeShipper_address= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_address');
				$wc_gls_AlternativeShipper_city= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_city');
				$wc_gls_AlternativeShipper_zipcode= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_zipcode');
				$wc_glsAlternativeShipper_country= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_glsAlternativeShipper_country');
				$wc_gls_AlternativeShipper_contact= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_contact');
				$wc_gls_AlternativeShipper_phone= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_phone');
				$wc_gls_AlternativeShipper_mobile= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_mobile');
				$wc_gls_AlternativeShipper_email= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_AlternativeShipper_email');
			}
			
			$wc_processing_time= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_diverse_txt'].'_wc_processing_time',0);

			if( $wc_processing_time > 0 ){
				$ShipmentDate = new DateTime('+'.$wc_processing_time.' day');
				$ShipmentDate = $ShipmentDate->format('Ymd');
			}else{
				$ShipmentDate = date('Ymd');
			}
			
			$wc_gls_testmode = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_txt'].'_wc_gls_testmode','on');
			if($wc_gls_testmode=='on'){	
				$gls_username = "";
				$gls_password = "";
				$gls_Customerid = "";
				$gls_contactid = "";
			}
			else{	
				$gls_username = get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_username');
				$this->log_me("gls_username: $gls_username");
				if( ! $gls_username ){
					die();
				}
				$gls_password= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_password');
				$gls_Customerid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_Customerid');
				$gls_contactid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_contactid');
			}
			
			$parceldata = array();
			$parceldata["UserName"] = $gls_username;
			$parceldata["Password"] = $gls_password;
			$parceldata["Customerid"] = $gls_Customerid;
			$parceldata["Contactid"] = $gls_contactid;
			$parceldata["ShipmentDate"] = $ShipmentDate;
			$parceldata["Reference"] = 'fak info:'.$order->get_billing_email().' '.$order->get_billing_phone();//Note 1
			$parceldata["Addresses"]["Delivery"]["City"] = $billing_city;
			$parceldata["Addresses"]["Delivery"]["Email"] = $billing_email;
			$parceldata["Addresses"]["Delivery"]["Contact"] = $billing__contact;
			$parceldata["Addresses"]["Delivery"]["Mobile"] = $billing_phone;
			$parceldata["Addresses"]["Delivery"]["CountryNum"] = '208';
			$parceldata["Addresses"]["Delivery"]["Name1"] = $billing_name;
			$parceldata["Addresses"]["Delivery"]["Street1"] = $billing_address_1;
			$parceldata["Addresses"]["Delivery"]["ZipCode"] = $billing_postcode;
			$parceldata["Parcels"][0]["Weight"] = $total_weight;
		
			if( $wc_gls_AlternativeShipper == 'on' ){
				$parceldata["Addresses"]["AlternativeShipper"]["Name1"] = $wc_gls_AlternativeShipper_name;
				$parceldata["Addresses"]["AlternativeShipper"]["Street1"] = $wc_gls_AlternativeShipper_address;
				$parceldata["Addresses"]["AlternativeShipper"]["CountryNum"] = "208";
				$parceldata["Addresses"]["AlternativeShipper"]["ZipCode"] = $wc_gls_AlternativeShipper_zipcode;
				$parceldata["Addresses"]["AlternativeShipper"]["City"] = $wc_gls_AlternativeShipper_city;
				$parceldata["Addresses"]["AlternativeShipper"]["Contact"] = $wc_gls_AlternativeShipper_contact;
				$parceldata["Addresses"]["AlternativeShipper"]["Email"] = $wc_gls_AlternativeShipper_email;
				$parceldata["Addresses"]["AlternativeShipper"]["Phone"] = $wc_gls_AlternativeShipper_phone;
				$parceldata["Addresses"]["AlternativeShipper"]["Mobile"] = $wc_gls_AlternativeShipper_mobile;
			}

			switch( $parcelshop_idx ){
				case 0:
					$parceldata["Services"]["ShopDelivery"] = $parcelshop_number;
					$parceldata["Services"]["NotificationEmail"] = $billing_email;
					$parceldata["Parcels"][0]["Reference"] = __('Order ID: ',$this->plugin_name).$order_id;//Ref. Nr.
					if( $ship_to_different_address == "shipping_"){
						$s_txt="";			
						$s_email=$order->get_meta( 'shipping_email' );
						if ( ! empty($s_email) )
							$s_txt=$s_email;    
						$s_phone=$order->get_meta( 'shipping_phone' );
						if ( ! empty($s_phone) )
							$s_txt=$s_txt.' '.$s_phone;
						if ( ! empty($s_txt) )			
							$parceldata["Parcels"][0]["Comment"] = "Lev info:".$s_txt;//Note 2
					}else{
					}
					break;
				case 1:
					break;
				case 2:
					if( $parcelshop_message != "" ){
						$parceldata["Services"]["Deposit"] = $parcelshop_message;
					}
					break;
				case 3:
					break;
			}
			$url = "https://blohm.com/jwt/gls_parcel_shop_server.php";
			$data=[
				'UserName' => $gls_username,
				'Password' => $gls_password,
				'Customerid' => $gls_Customerid,
				'Contactid' => $gls_contactid,
				'action'=>'ret_token',
				'timezone'=>'',
				'testmode'=>$wc_gls_testmode,
				'country'=>'',
				'address'=>'',
				'zip_code'=>'',
				'amount'=>'',
				'number'=>'',
				'parcel_data'=>$parceldata,
				'token'=>'',
			];
			$data = json_encode($data);

			$data_1 = wp_remote_post($url, array(
				'method' => 'POST',
				'headers' => array(
					'content-Type' => 'application/json; charset=utf-8'
				),
				'timeout'     => 60, // added
				'redirection' => 5,  // added
				'blocking'    => true, // added
				'httpversion' => '1.0',
				'sslverify' => false,
				'body' => $data
			));

			if( is_wp_error( $data_1 ) ) {
				echo "error";
				exit();	
			}

			$response = wp_remote_retrieve_body( $data_1 );
			
			$data=[
				'UserName' => $gls_username,
				'Password' => $gls_password,
				'Customerid' => $gls_Customerid,
				'Contactid' => $gls_contactid,
				'action'=>'ajax_gls_get_label',
				'timezone'=>'',
				'testmode'=>$wc_gls_testmode,
				'country'=>'',
				'address'=>'',
				'zip_code'=>'',
				'amount'=>'',
				'number'=>'',
				'parcel_data'=>$parceldata,
				'token'=>$response,
			];
			$data = json_encode($data);

			$data_1 = wp_remote_post($url, array(
				'method' => 'POST',
				'headers' => array(
					'content-Type' => 'application/json; charset=utf-8'
				),
				'timeout'     => 60, // added
				'redirection' => 5,  // added
				'blocking'    => true, // added
				'httpversion' => '1.0',
				'sslverify' => false,
				'body' => $data
			));

			if( is_wp_error( $data_1 ) ) {
				echo "error";
				exit();	
			}

			$response = wp_remote_retrieve_body( $data_1 );
			$this->log_me($response);
			
			if($response == true && isset(json_decode($response)->PDF))
			{
				$gls_order_tracking_number=json_decode($response)->Parcels[0]->ParcelNumber;
				update_post_meta( $order_id, 'gls_order_tracking_number', $gls_order_tracking_number );
				$pdf_decoded = json_decode($response);
				$order->update_meta_data('_gls_label', $pdf_decoded);
                $order->save();
			}else{
				if(isset(json_decode($response)->Message)){
					echo '<div class="error" style="color:red;">';
					echo json_decode($response)->Message;
					echo '</div>';
					die();
				}else{
					echo '<div class="error" style="color:red;"> Please check credentails or data is invalid. </div>';
					die();
				}
			}							
		}
	}	

	/**
	 * Auto generate one shipping zone with one flatrate and one local pick up if not exists
	 * @return void
	 */
	public function setup_gls_shippings_zones() {
			
		$country_code = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_txt'].'_wc_gls_Shipper_country','DK');	
		//$this->log_me("country_code: $country_code");

		$shipping_methods_id = "";
		$instance_id	   = false;	
		$zone_name     = 'GLS WC '.$country_code;
		$gls_shipping_found='no';;
		$gls_local_found='no';
		$first_id=true;
		$first_title=false;
		$gls_zone_id=-1;
		
		$flat_rate     = 'flat_rate';
		$local_pickup = 'local_pickup';
									
		$tax_status = 'taxable';
		$cost = '40';
		
		$zones = WC_Shipping_Zones::get_zones();
		if (!empty($zones)) {
			foreach ($zones as $zone_id => $zone_data) {
				$zone = WC_Shipping_Zones::get_zone($zone_id);
				$location=$zone->get_zone_locations();
				$zone_methods = $zone->get_shipping_methods();
				if (!empty($zone_methods)) {
					foreach ($zone->get_shipping_methods() as $instance_id => $method) {
						if ( 'country' === $location[0]->type && $country_code === $location[0]->code && $method->enabled ){
							if($first_id){
								$gls_zone_id=$zone_id;
								$first_id=false;
							}
							if( $method->id === $flat_rate ){
								$gls_shipping_found='yes';
								
								$first_title=true;
								foreach ($zone_methods as $instance_id_1 => $method_1) {
									if( $method_1->id === $local_pickup && $method_1->enabled ){
										$gls_local_found='yes';
										break;
									}
								}
								break;
							}
						}
					}
				}
				if( $first_title )
					break;
			}
		}
	
		if( $gls_shipping_found == 'no'){
			$zone_add = null;
			if ($gls_zone_id === -1) {	
				 
				// Instantiate a new shipping zone with our object
				$zone_add = new WC_Shipping_Zone();
				$zone_add->set_zone_name( $zone_name );

				// Add Contry as location
				$zone_add->add_location( $country_code, 'country' );

				// Save the zone, if non existent it will create a new zone
				$zone_add->save();
			}
			$m_local_pickup='no';
			if( $zone_add === null )
				$zone_add = WC_Shipping_Zones::get_zone($gls_zone_id);
						
			$instance_id_add = $zone_add->add_shipping_method($flat_rate);
			add_option( 'woocommerce_'. $flat_rate .'_'. $instance_id_add .'_settings', array(
				'tax_status' => $tax_status,
				'cost'       => $cost
			),'' , 'yes' );	

			$zone_methods = $zone_add->get_shipping_methods();
			if (!empty($zone_methods)) {
				foreach ($zone_add->get_shipping_methods() as $instance_id => $method) {							
					if( $method->id === $local_pickup ){
						$m_local_pickup='yes';
					}
				}
			}
			
			if( $m_local_pickup === 'no' ){
				$instance_id_add = $zone_add->add_shipping_method($local_pickup);
				add_option( 'woocommerce_'. $local_pickup .'_'. $instance_id_add .'_settings', array(
				'tax_status' => ''
				),'' , 'yes' );	
			}	
		}else
		if( $gls_local_found === 'no' ){
			$zone_add = WC_Shipping_Zones::get_zone($gls_zone_id);
			$instance_id_add = $zone_add->add_shipping_method($local_pickup);
			add_option( 'woocommerce_'. $local_pickup .'_'. $instance_id_add .'_settings', array(
			'tax_status' => ''
			),'' , 'yes' );	
		}
	}
	
	/**
	 * Recursive sanitation for text or array
	 * 
	 * @param $array_or_string (array|string)
	 * @since  0.1
	 * @return mixed
	 */
	function sanitize_text_or_array_field($array_or_string) {
		if( is_string($array_or_string) ){
			$array_or_string = sanitize_text_field($array_or_string);
		}elseif( is_array($array_or_string) ){
			foreach ( $array_or_string as $key => &$value ) {
				if ( is_array( $value ) ) {
					$value = sanitize_text_or_array_field($value);
				}
				else {
					$value = sanitize_text_field( $value );
				}
			}
		}

		return $array_or_string;
	}	
	
	public function wc_action_callback_gls() {

		if (isset($_POST['message'])) {
			$ar=$_POST['message'];
			$ar=$this->sanitize_text_or_array_field($ar);
			$points = json_decode(stripslashes($ar));
			$wc_gls_testmode = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_txt'].'_wc_gls_testmode','on');

			if($wc_gls_testmode=='on'){	
				$gls_username = "";
				$gls_password = "";
				$gls_Customerid = "";
				$gls_contactid = "";
			}
			else{	
				$gls_username = get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_username');
				$this->log_me("gls_username: $gls_username");
				if( ! $gls_username ){
					die();
				}
				$gls_password= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_password');
				$gls_Customerid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_Customerid');
				$gls_contactid= get_option($this->base.$this->plugin_name.'_'.$this->plugin_data['get_gls_tab_txt'].'_wc_gls_contactid');
			}
			
			$wc_wc_display_shops = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_diverse_txt'].'_wc_display_shops','5');

			$url = "https://blohm.com/jwt/gls_parcel_shop_server.php";

			if($points->name == "ajax_gls_chosen_shipping_methods"){
				
				$chosen  = WC()->session->get('chosen_shipping_methods'); // The chosen methods
							
				$shipping_postcode = WC()->customer->get_shipping_postcode();
				$billing_postcode = WC()->customer->get_billing_postcode();
				$postcode = ! empty($shipping_postcode) ? $shipping_postcode : $billing_postcode;
							
				$shipping_country = WC()->customer->get_shipping_country();
				$billing_country = WC()->customer->get_billing_country();
				$country = ! empty($shipping_country) ? $shipping_country : $billing_country;
				
				$shipping_city = WC()->customer->get_shipping_city();
				$billing_city = WC()->customer->get_billing_city();
				$city = ! empty($shipping_city) ? $shipping_city : $billing_city;
		
				$json = array();	
				$json['chosen'] =$chosen;
				$json['postcode'] =$postcode;
				$json['country'] =$country;
				$json['city'] =$city;
				$encoded = json_encode($json);
				echo $encoded;
			}else
			if($points->name == "ajax_gls_shipp"){
				
				WC()->session->set( '_werner_shop_idx', $points->idx );
		
				$json = array();	
				$json['idx'] =$points->idx;
				$encoded = json_encode($json);
				echo $encoded;
			}else
			if($points->name == "ajax_gls_get_shops"){
				$country=$points->country;
				$post_code=$points->post_code;
				$billing_address=$points->billing_address;
				$wc_gls_timezone=get_option('timezone_string');
				$amount=$wc_wc_display_shops;
				$data=[
					'UserName' => $gls_username,
					'Password' => $gls_password,
					'Customerid' => $gls_Customerid,
					'Contactid' => $gls_contactid,
					'action'=>'ret_token',
					'timezone'=>$wc_gls_timezone,
					'testmode'=>$wc_gls_testmode,
					'country'=>$country,
					'address'=>$billing_address,
					'zip_code'=>$post_code,
					'amount'=>$amount,
					'number'=>'',
					'parcel_data'=>'',
					'token'=>'',
				];
				$data = json_encode($data);

				$data_1 = wp_remote_post($url, array(
					'method' => 'POST',
					'headers' => array(
						'content-Type' => 'application/json; charset=utf-8'
					),
					'timeout'     => 60, // added
					'redirection' => 5,  // added
					'blocking'    => true, // added
					'httpversion' => '1.0',
					'sslverify' => false,
					'body' => $data
				));

				if( is_wp_error( $data_1 ) ) {
					echo "error";
					exit();	
				}

				$response = wp_remote_retrieve_body( $data_1 );
				
				$data=[
					'UserName' => $gls_username,
					'Password' => $gls_password,
					'Customerid' => $gls_Customerid,
					'Contactid' => $gls_contactid,
					'action'=>'ajax_gls_get_shops',
					'timezone'=>$wc_gls_timezone,
					'testmode'=>$wc_gls_testmode,
					'country'=>$country,
					'address'=>$billing_address,
					'zip_code'=>$post_code,
					'amount'=>$amount,
					'number'=>'',
					'parcel_data'=>'',
					'token'=>$response,
				];
				$data = json_encode($data);

				$data_1 = wp_remote_post($url, array(
					'method' => 'POST',
					'headers' => array(
						'content-Type' => 'application/json; charset=utf-8'
					),
					'timeout'     => 60, // added
					'redirection' => 5,  // added
					'blocking'    => true, // added
					'httpversion' => '1.0',
					'sslverify' => false,
					'body' => $data
				));

				if( is_wp_error( $data_1 ) ) {
					echo "error";
					exit();	
				}

				$response = wp_remote_retrieve_body( $data_1 );
				echo $response;
			}else
			if($points->name == "ajax_gls_number"){
				$shop_number=$points->shop_number;
				$country=$points->country;	
				$wc_gls_timezone=get_option('timezone_string');
				
				$data=[
					'UserName' => $gls_username,
					'Password' => $gls_password,
					'Customerid' => $gls_Customerid,
					'Contactid' => $gls_contactid,
					'action'=>'ret_token',
					'timezone'=>$wc_gls_timezone,
					'testmode'=>$wc_gls_testmode,
					'country'=>$country,
					'address'=>'',
					'zip_code'=>'',
					'amount'=>'',
					'number'=>$shop_number,
					'parcel_data'=>'',
					'token'=>'',
				];
				$data = json_encode($data);


				$data_1 = wp_remote_post($url, array(
					'method' => 'POST',
					'headers' =>  array(
						'content-Type' => 'application/json; charset=utf-8'
					),
					'timeout'     => 60, // added
					'redirection' => 5,  // added
					'blocking'    => true, // added
					'httpversion' => '1.0',
					'sslverify' => false,
					'body' => $data
				));

				if( is_wp_error( $data_1 ) ) {
					echo "error";
					exit();	
				}

				$response = wp_remote_retrieve_body( $data_1 );
				
				$data=[
					'UserName' => $gls_username,
					'Password' => $gls_password,
					'Customerid' => $gls_Customerid,
					'Contactid' => $gls_contactid,
					'action'=>'verify_number',
					'timezone'=>$wc_gls_timezone,
					'testmode'=>$wc_gls_testmode,
					'country'=>$country,
					'address'=>'',
					'zip_code'=>'',
					'amount'=>'',
					'number'=>$shop_number,
					'parcel_data'=>'',
					'token'=>$response,
				];
				$data = json_encode($data);


				$data_1 = wp_remote_post($url, array(
					'method' => 'POST',
					'headers' =>  array(
						'content-Type' => 'application/json; charset=utf-8'
					),
					'timeout'     => 60, // added
					'redirection' => 5,  // added
					'blocking'    => true, // added
					'httpversion' => '1.0',
					'sslverify' => false,
					'body' => $data
				));

				if( is_wp_error( $data_1 ) ) {
					echo "error";
					exit();	
				}

				$response = wp_remote_retrieve_body( $data_1 );
				//$result = json_decode( $response, true );
				echo $response;
			}
		}
		// Always exit to avoid further execution
		exit();		
	}
	
	/**
	 * Add Shipping Options before Order Review in WooCommerce Checkout if any items
	 * in cart require shipping.
	 */
	public function pwb_gls_delivery_options(){
		?>
		<style>	
		#map {
		width : 100%;
		height: 400px;
		}
		</style>

		<!-- Accordion -->
		<div class="container-fluid bg-gray" id="accordion-style-1">
			<div class="container-fluid">
				<section>
					<div class="row">
						<div class="col-12">
							<h1 class="pwb-prefix-class text-green mb-4 text-center"><?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'c_headline_checkout',__( 'Select shipping method', $this->plugin_name )); ?></h1>
						</div>
						<div class="col-12 mx-auto">
							<div class="accordion" id="accordionExample">
									
								<div class="pwb-prefix-class accordion-item" id="pickup_parcelshop_card">
									<div class="card">
										<div class="card-header" id="headingOne">
											<h5 class="mb-0">
												<button class="btn btn-link btn-block text-left" type="button" data-toggle="collapse" data-target="#collapseOne" aria-expanded="true" aria-controls="collapseOne">
												<i class="fa fa fa-truck main"></i><i class="fa fa-angle-double-right mr-3"></i>
												<?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_pickup_parcelshop',__( 'GLS. Pick up at parcelshop', $this->plugin_name )); ?>
												</button>
											</h5>
										</div>
										<div id="collapseOne" class="collapse fade collapse_idx" aria-labelledby="headingOne" data-parent="#accordionExample">
											<div class="card-body">
																				
												<div class="pwb-prefix-class wrapper_gls" id="full-width-page-wrapper_gls">
													<div class="pwb-prefix-class container-fluid" id="content">
														<div id="maindiv">		
															<div id="map" class="pwb-prefix-class map map-home" style="height: 300px; margin-top: 50px"></div>
														</div>
														<div class="pwb-prefix-class row">
															<div class="pwb-prefix-class col-md-12 content-area" id="primary">
																<div class="pwb-prefix-class container-fluid">
																	<div class="pwb-prefix-class row-fluid">
																		<div class="pwb-prefix-class rov"><!--
																			<h2><p id="scroll1"><?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_headline_post_shopnumber',__( 'Alternative shops. Enter Post or Parcel shop number', $this->plugin_name )); ?></p></h2>
																			<div class="pwb-prefix-class col-6">
																			</div>
																			<div class="pwb-prefix-class row">
																				<div class="pwb-prefix-class col-sm-6">
																					<div class="pwb-prefix-class form-group">
																						<input id="postnumber" type="number" class="pwb-prefix-class form-control mt-2" placeholder="POST NUMMER" style="width:200px;text-align: center;" max="4" min="0" step="1">
																					</div>
																				</div>
																				<div class="pwb-prefix-class col-sm-6">
																					<div class="pwb-prefix-class form-group">
																					<input id="shop_number" type="number" class="pwb-prefix-class form-control mt-2" placeholder="SHOP NUMMER" style="width:200px;text-align: center;" max="5" min="0" step="1">
																					</div>
																				</div>
																			</div>
																			-->
																			<address>
																				<div class="pwb-prefix-class accordion" id="accordion_gls">
																					<div id="GlsResults"></div>
																				</div>
																			</address>
																		</div>
																	</div>
																</div>
															</div><!-- #primary -->
														</div><!-- .row end -->
													</div><!-- #content -->
												</div><!-- #full-width-page-wrapper -->	
												
											</div>
										</div>
									</div>
								</div>
							
								<div class="pwb-prefix-class accordion-item" id="business_delevery_card">
									<div class="card">
										<div class="card-header" id="headingTwo">
											<h5 class="mb-0">								
												<button class="btn btn-link collapsed btn-block text-left" type="button" data-toggle="collapse" data-target="#collapseTwo" aria-expanded="false" aria-controls="collapseTwo">
												 <i class="fa fa-truck main"></i><i class="fa fa-angle-double-right mr-3"></i>
												 <?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_business_delevery',__( 'GLS. Delivery to business address', $this->plugin_name )); ?>
												</button>
											</h5>
										</div>
										<div id="collapseTwo" class="collapse fade collapse_idx" aria-labelledby="headingTwo" data-parent="#accordionExample">
											<div class="card-body">

												<?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_profession_text_block',__( 'GLS Delivery to business address<br>
												Weekdays between 7-16.</br>
												GLS delivers for the goods delivery together with the company other packages.</br>', $this->plugin_name )); ?>
												
											</div>
										</div>
									</div>
								</div>			
								
								<div class="pwb-prefix-class accordion-item" id="home_delevery">
									<div class="card">
										<div class="card-header" id="headingThree">
											<h5 class="mb-0">
												<button class="btn btn-link collapsed btn-block text-left" type="button" data-toggle="collapse" data-target="#collapseThree" aria-expanded="false" aria-controls="collapseThree">
												  <i class="fa fa-truck main"></i><i class="fa fa-angle-double-right mr-3"></i>
												  <?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_home_delivery',__( 'Home delivery', $this->plugin_name )); ?>
												</button>
											</h5>
										</div>
										<div id="collapseThree" class="collapse fade collapse_idx" aria-labelledby="headingThree" data-parent="#accordionExample">
											<div class="card-body">
											
												<!--<label for="exampleDropdownFormEmail2" class="pwb-prefix-class form-label"><strong><?php echo __( 'Message to transporter', $this->plugin_name ); ?></strong></label>
												<input type="text" type="hidden" class="pwb-prefix-class form-control" id="message_to_transporter" placeholder="<?php echo __("Place the parcel in garage", $this->plugin_name ); ?>">-->
												<?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_home_delevery_text_block',__( '
												GLS Home Delivery - Delivery to your address<br> 
												Weekdays between 7-16.<br> 
												If GLS does not reach anyone at the address, you will receive information about what happens to your package.<br> 
												This information is either a note in your mailbox or an email / text message.<br> 
												There are two options:<br> 
												1. GLS runs the package to a parcel shop nearby.<br> 
												You bring package no. and ID and retrieve the package when it suits you.<br> 
												2. GLS takes the package back to GLS Depot.<br> 
												You must then tell how you want the delivery<br> 
												- Do you want to pick up at the depot ?.<br> 
												-Should the package be redirected to your workplace ?.<br> 
												-Should GLS deliver to the original address ?.<br> 
												<br> 
												In either case, you have 10 business days to respond.<br> 
												The GLS then returns the package to the sender.<br> ', $this->plugin_name ));?>	
												
											</div>
										</div>
									</div>
								</div>
								
								<div class="pwb-prefix-class accordion-item" id="pickup_at_firm">
									<div class="card">
										<div class="card-header" id="headingFour">
											<h5 class="mb-0">
												<button class="btn btn-link collapsed btn-block text-left" type="button" data-toggle="collapse" data-target="#collapseFour" aria-expanded="false" aria-controls="collapseFour">
												  <i class="fa fa-truck main"></i><i class="fa fa-angle-double-right mr-3"></i>							  
												  <?php echo get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_checkout'].'_wc_pickup_at_videx',__( 'Pick up at our company address', $this->plugin_name )); ?>
												</button>
											</h5>
										</div>
										<div id="collapseFour" class="collapse fade collapse_idx" aria-labelledby="headingFour" data-parent="#accordionExample">
											<div class="card-body">

												<div class="pwb-prefix-class business-hours">
													<h2 class="pwb-prefix-class title m-1"><?php echo __( 'Opening Hours', $this->plugin_name ); ?></h2>
													<?php 
														$Sunday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Sunday_start','');
														$Sunday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Sunday_end','');
														if( $Sunday_start == "" || $Sunday_end == "" ) 
															$Sunday=__( 'Closed' , $this->plugin_name );
														else
															$Sunday=$Sunday_start.":".$Sunday_end;
															
														$Monday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Monday_start','8:00');
														$Monday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Monday_end','16:00');
														if( $Monday_start == "" || $Monday_end == "" ) 
															$Monday=__( 'Closed' , $this->plugin_name );
														else
															$Monday=$Monday_start.":".$Monday_end;
															
														$Tuesday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Tuesday_start','8:00');
														$Tuesday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Tuesday_end','16:00');
														if( $Tuesday_start == "" || $Tuesday_end == "" ) 
															$Tuesday=__( 'Closed' , $this->plugin_name );
														else
															$Tuesday=$Tuesday_start.":".$Tuesday_end;
															
														$Wednesday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Wednesday_start','8:00');
														$Wednesday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Wednesday_end','16:00');
														if( $Wednesday_start == "" || $Wednesday_end == "" ) 
															$Wednesday=__( 'Closed' , $this->plugin_name );
														else
															$Wednesday=$Wednesday_start.":".$Wednesday_end;
															
														$Thursday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Thursday_start','8:00');
														$Thursday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Thursday_end','16:00');
														if( $Thursday_start == "" || $Thursday_end == "" ) 
															$Thursday=__( 'Closed' , $this->plugin_name );
														else
															$Thursday=$Thursday_start.":".$Thursday_end;
															
														$Friday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Friday_start','8:00');
														$Friday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Friday_end','16:00');
														if( $Friday_start == "" || $Friday_end == "" ) 
															$Friday=__( 'Closed' , $this->plugin_name );
														else
															$Friday=$Friday_start.":".$Friday_end;
															
														$Saturday_start = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Saturday_start','');
														$Saturday_end = get_option($this->base.$this->plugin_name."_".$this->plugin_data['get_gls_tab_shop_time'].'_Saturday_end','');
														if( $Saturday_start == "" || $Saturday_end == "" ) 
															$Saturday=__( 'Closed' , $this->plugin_name );
														else
															$Saturday=$Saturday_start.":".$Saturday_end;
													?>
													<ul class="pwb-prefix-class list-unstyled opening-hours">
													<li><?php echo __( 'Sunday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Sunday; ?></span></li>
													<li><?php echo __( 'Monday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Monday; ?></span></li>
													<li><?php echo __( 'Tuesday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Tuesday; ?></span></li>
													<li><?php echo __( 'Wednesday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Wednesday; ?></span></li>
													<li><?php echo __( 'Thursday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Thursday; ?></span></li>
													<li><?php echo __( 'Friday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Friday; ?></span></li>
													<li><?php echo __( 'Saturday', $this->plugin_name ); ?> <span class="pwb-prefix-class pull-right"><?php echo $Saturday; ?></span></li>
													</ul>
												</div>								
										
											</div>
										</div>
									</div>
								</div>
								
							</div>
						</div>	
					</div>
				</section>
				<input id="parcelshop_idx" name="properties[parcelshop_idx]" value="0" type="hidden"/>
				<input id="parcelshop_number" name="properties[parcelshop_number]" value="nob" type="hidden"/>
				<input id="parcelshop_message" name="properties[parcelshop_message]" value="nob" type="hidden"/>
				<input id="ship_to_different_address" name="properties[ship_to_different_address]" value="billing_" type="hidden"/>
				
				<input type="hidden" id="no_api" value="<?php echo __( 'Api credentials are not correct, please fill in the correct values.GLS (Customer ID)/(Contact ID)', $this->plugin_name ); ?>">
				<input type="hidden" id="opening_hours" value="<?php echo __( 'Opening Hours', $this->plugin_name ); ?>">
				<input type="hidden" id="Closed" value="<?php echo __( 'Closed' , $this->plugin_name ); ?>">
				<input type="hidden" id="Sunday" value="<?php echo __( 'Sunday', $this->plugin_name ); ?>">
				<input type="hidden" id="Monday" value="<?php echo __( 'Monday', $this->plugin_name ); ?>">
				<input type="hidden" id="Tuesday" value="<?php echo __( 'Tuesday', $this->plugin_name ); ?>">
				<input type="hidden" id="Wednesday" value="<?php echo __( 'Wednesday', $this->plugin_name ); ?>">
				<input type="hidden" id="Thursday" value="<?php echo __( 'Thursday', $this->plugin_name ); ?>">
				<input type="hidden" id="Friday" value="<?php echo __( 'Friday', $this->plugin_name ); ?>">
				<input type="hidden" id="Saturday" value="<?php echo __( 'Saturday', $this->plugin_name ); ?>">
			</div>
		</div>
		<!-- .// Accordion -->
		<?php	
	}	

}
